"""
Tests for the functionality of the Notification REST API.
The logic can be found in `notification/views/notify/py`.
"""
import http
from unittest.mock import patch

import notification.views.notify
import pytest
from pyramid.testing import DummyRequest

from workspaces.notification.services.notification_service import NotificationService


@pytest.mark.usefixtures("dummy_send", "mock_notification_service")
class TestNotifyViews:
    def test_view_notifications(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationListRestService

        response = NotificationListRestService(dummy_send).list_notification_templates()
        assert len(response) == 1

    def test_send_notification(
        self, dummy_send: DummyRequest, mock_notification_service: NotificationService
    ):
        from notification.views.notify import NotificationRestService

        dummy_send.json_body = {
            "destination_email": "chausman@nrao.edu",
            "subject": "first thing",
            "message": "second thing",
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch(
            "workspaces.notification.services.notification_service.NotificationService.send_email"

        ) as send_email:
            send_email.return_value = {}
            response = NotificationRestService(dummy_send).send_notification()
        assert response['errors'] == {}
        assert response['message'] == 'Email sent'

    def test_create_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationListRestService

        dummy_send.json_body = {
            "name": "new_template",
            "description": "a test template",
            "content": "Subject: {{foo}}\n\n{{bar}}",
        }

        response = NotificationListRestService(dummy_send).create_notification_template()
        assert response.status_code == http.HTTPStatus.CREATED
        assert response.json_body == dummy_send.json_body

    def test_delete_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.matchdict["template_name"] = "new_template"

        response = NotificationRestService(dummy_send).delete_notification_template()
        assert response.status_code == http.HTTPStatus.OK
